/**
 * test-rxnav-api.js
 * 
 * shows examples for most, if not all, services
 * 
 */
const RxnavApi = require('../rxnav-api')
const { UmlsApi } = require('umls-api')
const fs = require("fs")
const os = require('os')
const moment = require('moment')

const allConceptsByTermTypeList = () => {
    let types = ['BPCK']
    types = ['GPCK']
    // types = ['BN']
    const rxnavApi = new RxnavApi()
    rxnavApi.json().rxnorm().allConceptsByTermTypeList(types)//['BN','BPCK'])
        .then((res) => {
            if (rxnavApi.isJson()) {
                console.log(`${JSON.stringify(res)}`)
                // console.log(`No. ${types} : ${res.minConceptGroup.minConcept.length}`)
            } else {
                console.log(`xml (not json) so dumping all xml\n\n${res}`)
            }
        }
        )
        .catch(err => console.log(`err=${err}`))

}
// allConceptsByTermTypeList()

const approximateTerm = () => {
    new RxnavApi().xml().rxnorm().approximateTerm({ term: 'zocor 10 mg', option: 1, maxEntries: 3 })
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// approximateTerm()
const brandsContainingIngredients = () => {
    new RxnavApi().xml().rxnorm().brandsContainingIngredients([8896, 20610])
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// brandsContainingIngredients()
const displayNames = () => {
    new RxnavApi().json().rxnorm().displayNames()
        .then((res) =>
            console.log(res))
        .catch(err => console.log(`err=${err}`))
}
// displayNames()

const drugs = () => {
    let searchTerm = 'cymbalta'
    searchTerm = 'warfarin'
    searchTerm = 'Warfarin Sodium 1 MG'
    //searchTerm = 'cannabis'
    new RxnavApi().json().rxnorm().drugs(searchTerm)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
//drugs()
const idTypeList = () => {
    new RxnavApi().json().rxnorm().idTypeList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// idTypeList()

const ndcProperties = () => {
    let ndc = '1C5BC1DD-E9EC-44C1-9281-67AD482315D9'
    ndc = '213270'
    new RxnavApi().isJson().rxnorm().ndcProperties(ndc)//'00904629161') //'37000-653-04')//'0069-3150-83')
        .then((ndcinfo) =>
            console.log(JSON.stringify(ndcinfo)))
        .catch(err => console.log(`err=${err}`))
}
// ndcProperties()
const ndcStatus = () => {
    let ndc = '1C5BC1DD-E9EC-44C1-9281-67AD482315D9'
    ndc = '213270'
    ndc = '00143314501'
    let payload = {
        ndc: ndc,
        start: null,
        end: null,
        history: 0,
        altpkg: 0
    }
    new RxnavApi().json().rxnorm().ndcStatus(payload)
        .then((ndcStatusInfo) =>
            console.log(JSON.stringify(ndcStatusInfo)))
        .catch(err => console.log(`err=${err}`))
}
//ndcStatus()

const propertyCategoryList = () => {
    new RxnavApi().json().rxnorm().propertyCategoryList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// propertyCategoryList()

const propertyNameList = () => {
    new RxnavApi().json().rxnorm().propertyNameList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// propertyNameList()

const rxnormRelationTypeList = () => {
    new RxnavApi().json().rxnorm().relationTypeList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// rxnormRelationTypeList()

const restfulEndpointsList = () => {
    new RxnavApi().restfulEndpointsList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// restfulEndpointsList()


const rxnormSourceTypeList = () => {
    new RxnavApi().sourceTypeList()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
rxnormSourceTypeList()


const spellingSuggestionList = () => {
    new RxnavApi().xml().rxnorm().spellingSuggestionList('ambien')
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// spellingSuggestionList()

const termTypes = () => {
    new RxnavApi().xml().rxnorm().termTypes()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// termTypes()

const rxNormVersion = () => {
    new RxnavApi().xml().rxnorm().version()
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// rxNormVersion()

const ndcsForRxnorm = () => {
    new RxnavApi().xml().rxnorm().rxcui().ndcsForRxnorm(1668240)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
//ndcsForRxnorm()


/**
 * In the example below, the TTY property for morphine (RxCUI=7052) 
 * is checked to see if the concept is an ingredient.
 */
const doesRxnormHaveProperty = () => {
    let morphineRxnorm = 7052
    new RxnavApi().xml().rxnorm().doesRxnormHaveProperty(morphineRxnorm, { propName: 'TTY', propValues: ['IN'] })
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// doesRxnormHaveProperty()



const rxnormsForOtherVocabulary = () => {
    let termType = 'GCN_SEQNO'
    const gcnSeqno = '009172'
    let otherVocabularyCode = gcnSeqno
    // termType = 'NDC'
    // otherVocabularyCode = '0781-1506-10'
    // termType = 'NUI'
    // otherVocabularyCode = 'N0000189366'
    new RxnavApi().xml().rxnorm().rxnormsForOtherVocabulary(termType, otherVocabularyCode)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// rxnormsForOtherVocabulary()

const rxnormsForFdbGcnSeqno = () => {
    let gcnSeqno = '009172'
    new RxnavApi().rxnorm().rxnormsForFdbGcnSeqno(gcnSeqno)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormsForFdbGcnSeqno()

const rxnormsForFdbIngredientId = () => {
    let fdbIngredientId = '004489'
    new RxnavApi().rxnorm().rxnormsForFdbIngredientId(fdbIngredientId)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//rxnormsForFdbIngredientId()

const rxnormsForHcpcsCode = () => {
    let hcpcsCode = 'J2710'
    new RxnavApi().rxnorm().rxnormsForHcpcsCode(hcpcsCode)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//rxnormsForHcpcsCode()

const rxnormsForSnomedCode = () => {
    let snomedCode = '372767007'// '1039008'
    new RxnavApi().rxnorm().rxnormsForSnomedCode(snomedCode)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//rxnormsForSnomedCode()
const rxnormsForNdc = () => {
    let ndc = '11523-7020-1'    //'00904629161'
    new RxnavApi().rxnorm().rxnormsForNdc(ndc)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//rxnormsForNdc()

const rxnormStatus = () => {
    let rxnorm = '105048'
    rxnorm = '44'
    // rxnorm = '1805014'
    new RxnavApi().xml().rxnorm().rxnormStatus(rxnorm)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormStatus()

const rxnormProperties = () => {
    let rxnorm = '131725' // ambien
    new RxnavApi().xml().rxnorm().rxnormProperties(rxnorm)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormProperties()

const rxnormProperty = () => {
    let rxnorm = '7052' // morphine
    let property = 'ATC'
    new RxnavApi().xml().rxnorm().rxnormProperty(rxnorm, property)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormProperty()


const rxnormNdcList = () => {
    let rxnorm = '213269' // Viagra 25 MG Oral Tablet 
    new RxnavApi().xml().rxnorm().rxnormNdcList(rxnorm)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormNdcList()

const loadCredentials = () => {
    //console.log(`hostname:${os.hostname()} : ${JSON.stringify(os.userInfo())}`)

    const homedir = os.homedir();
    const umlsFilePath = `${homedir}/.umls`
    if (fs.existsSync(umlsFilePath)) {
        const content = fs.readFileSync(umlsFilePath)
        const umlsCredentials = JSON.parse(content);
        umlsCredentials.createMoment = moment()
        return umlsCredentials
    } else {
        throw `unable to find file ${umlsFilePath}`
    }
}
const rxnormProprietaryInfo = () => {
    const umlsApi = new UmlsApi(loadCredentials())
    umlsApi.authenticate()
        .then((authenticatedUmlsApi) => {
            authenticatedUmlsApi.serviceTicket()
                .then((svcTicket) => {
                    let rxnorm = '261455' // Viagra 25 MG Oral Tablet 
                    rxnorm = '178'  // FDB Acetone
                    rxnorm = '2670' // FDB codeine
                    rxnorm = '197844' // ISOtretinoin_Claravis_20mg_cap
                    rxnorm = '197531'
                    new RxnavApi().xml().rxnorm().rxnormProprietaryInfo(rxnorm, ['MSH', 'RXNORM'], svcTicket)
                        .then((res) =>
                            console.log(JSON.stringify(res)))
                        .catch(err => console.log(`err=${err}`))

                })
                .catch((err) => { console.log(`DDDD, err=${err}`) })
        })
        .catch((err) => console.log(`OUCH,err=${err}`))

}
// rxnormProprietaryInfo()

const rxnormRelatedConceptList = () => {
    let rxnorm = '213269' // Viagra 25 MG Oral Tablet 
    new RxnavApi().json().rxnorm().rxnormRelatedConceptList(rxnorm)
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormRelatedConceptList()

const rxnormRelatedConceptListByRelationalAttributes = () => {
    let rxnorm = '174742' // Plavix
    // tradename_of+has_precise_ingredient
    new RxnavApi().json().rxnorm().rxnormRelatedConceptListByRelationalAttributes(rxnorm, ['tradename_of', 'has_precise_ingredient'])
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormRelatedConceptListByRelationalAttributes()


const rxnormRelatedConceptListByTermTypes = () => {
    let rxnorm = '174742' // Plavix
    // SBD+SBDF
    new RxnavApi().json().rxnorm().rxnormRelatedConceptListByTermTypes(rxnorm, ['SBD', 'SBDF'])
        .then((res) =>
            console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxnormRelatedConceptListByTermTypes()

const drugClassesById = () => {
    let classId = 'B01AA' // ATC id	ATC1-4
    new RxnavApi().json().rxclass().byId(classId)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassesById()

const drugClassesByName = () => {
    let className = 'radiopharmaceuticals'
    let classTypes = []
    new RxnavApi().json().rxclass().byName(className, classTypes)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassesByName()

const classTypeList = () => {
    new RxnavApi().json().rxclass().classTypeList()
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// classTypeList()

const drugClassSourceTypeList = () => {
    new RxnavApi().json().rxclass().sourceTypeList()
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassSourceTypeList()

const drugClassRelationTypeList = () => {
    let relaSources = ['dailymed']
    new RxnavApi().xml().rxclass().relationTypeList(relaSources)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassRelationTypeList()

const drugClassClassesByRxnorm = () => {
    let rxnorm = '174742' // Plavix
    rxnorm = '7052' // morphine
    let optionalSourceList = ['MEDRT']
    let optionalRelationshipList = [] // ['may_treat']
    new RxnavApi().json().rxclass().classesByRxnorm(rxnorm, optionalSourceList, optionalRelationshipList)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassClassesByRxnorm()

const drugClassesByDrugName = () => {
    let drugName = 'morphine'
    let optionalRelationshipList = ['may_treat']
    let optionalSourceList = ['MEDRT']
    new RxnavApi().json().rxclass().byDrugName(drugName, optionalSourceList, optionalRelationshipList)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// drugClassesByDrugName()

const drugClassesByClassTypeList = () => {
    let classTypeList = ['MOA']
    new RxnavApi().json().rxclass().byClassTypeList(classTypeList)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//drugClassesByClassTypeList()

const drugClassesClassContext = () => {
    let classId = 'D019275'
    new RxnavApi().json().rxclass().classContext(classId)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// drugClassesClassContext()

const drugClassesClassGraph = () => {
    let classId = 'D003879'
    let source = 'MESHPA'
    new RxnavApi().json().rxclass().classGraph(classId, source)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//drugClassesClassContext()

const drugClassesClassMembers = () => {
    let classId = 'N0000008638' // Decreased GnRH Secretion
    let source = 'DAILYMED'
    let relationship = 'has_PE'
    let optionalDirectIndirectFlag = 0 // both direct and indirect
    // optionalDirectIndirectFlag = 1 // direct only
    let termTypeList = ['IN', 'PIN']
    new RxnavApi().json().rxclass().classMembers(classId, source, relationship, termTypeList, optionalDirectIndirectFlag)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//drugClassesClassMembers()

const drugClassesClassTree = () => {
    const classId = 'N0000185505' // Cytochrome P450 Inducers
    new RxnavApi().json().rxclass().classTree(classId)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// drugClassesClassTree()

const drugClassesSimilarInfo = () => {
    const drugClass1 = {
        classId: 'N02AA', // natural opium alkaloids
        relaSource: 'ATC',
        rela: null
    }
    const drugClass2 = {
        classId: 'N0000008072', // morphine derivatives
        relaSource: 'NDFRT',
        rela: 'has_ingredient'
    }

    new RxnavApi().json().rxclass().similarInfo(drugClass1, drugClass2)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//drugClassesSimilarInfo()

const drugClassesSpellingSuggestions = () => {
    let term = 'marcolides'
    let type = 'CLASS' // 'DRUG' or 'CLASS'
    new RxnavApi().json().rxclass().spellingSuggestions(term, type)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// drugClassesSpellingSuggestions()

const drugClassesSimilarByRxnormList = () => {
    let rxnormList = ['7052', '7676', '7804', '23088']
    let sourceOfDrugRelations = 'DAILYMED'
    let relationship = 'has_EPC'
    let scoreType = 0  // optional; 0=default
    let numberOfResultsToReturn = 5
    let equivalenceThreshold = null // 0.0-1.0
    let inclusionThreshold = null // 0.0-1.0 [only relevant for scoreType=1-or-2]
    const payload = {
        rxcuis: rxnormList,
        relaSource: sourceOfDrugRelations,
        rela: relationship,
        scoreType,
        top: numberOfResultsToReturn,
        equivalenceThreshold,
        inclusionThreshold
    }
    new RxnavApi().json().rxclass().similarByRxnormList(payload)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// drugClassesSimilarByRxnormList()

const drugClassesSimilarToClass = () => {
    let classId = 'L01XE' // protein kinase inhibitors
    let sourceOfDrugRelations = 'ATC'
    let relationship = null //'has_EPC'
    let scoreType = 1  // optional; 0=default, 1=includes, 2=included_in
    let numberOfResultsToReturn = 2
    let equivalenceThreshold = null // 0.0-1.0
    let inclusionThreshold = null // 0.0-1.0 [only relevant for scoreType=1-or-2]

    const payload = {
        classId,
        relaSource: sourceOfDrugRelations,
        rela: relationship,
        scoreType,
        top: numberOfResultsToReturn,
        equivalenceThreshold,
        inclusionThreshold
    }
    new RxnavApi().json().rxclass().classesSimilarToClass(payload)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//drugClassesSimilarToClass()

const rxtermsVersion = () => {
    new RxnavApi().json().rxterms().version()
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
// rxtermsVersion()

const rxtermsAllConcepts = () => {
    new RxnavApi().json().rxterms().allConcepts()
        .then((res) => {
            // console.log(JSON.stringify(res))
            console.log(`Number found: ${res.minConceptGroup.minConcept.length}`)
        })
        .catch(err => console.log(`err=${err}`))

}
// rxtermsAllConcepts()

const rxtermsRxnormDisplayName = () => {
    let rxnorm = '198440' // Acetaminophen
    new RxnavApi().json().rxterms().rxnormDisplayName(rxnorm)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
// rxtermsRxnormDisplayName()

const rxtermsRxnormTerms = () => {
    let rxnorm = '198440' // Acetaminophen
    new RxnavApi().json().rxterms().rxnormTerms(rxnorm)
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
//rxtermsRxnormTerms()

const rximageSearch = () => {
    let rxnorm = '197698'
    let searchCriteria = {
        // image selection
        // size: 17,  // size in millimeters
        // sizeT : 0,  // size tolerance when searching by size
        // id : 185644464,
        // name : 'sertraline 100'
        // color:'blue',
        // imprint : 'teva,54',  // can have multiple things on it 
        // imprintColor : 'red',
        // inactive : 'gelatin',
        //ndc : '12345678901p3',
        // ndc : '11042-0142-03',
        //parse : 'blue+round' //+Limit=1',
        //shape : 'TRAPEZOID', //'DIAMOND', //'TRIANGLE',
        //matchRelabeled: false,  // used with ndc
        // matchPackSize : true,  // used with ndc  
        //rxnorm: '197698'  // not sure if this works - no results
        //score : 1, // number of sections the score breaks into
        symbol : false, // include or exclude pills with symbols or markings
        
        // results control
        rLimit : 1
    }
    new RxnavApi().json().rximage().search(searchCriteria,'rxbase')
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))
}
//rximageSearch()

const rximageSearchBySearchCriteria = () => {
    let searchCriteriaParams = new RxnavApi().rximage().createSearchCriteria()
    .color('blue+green')
    .color('brown')
    .limitNumberOfRecords(1)
    .params()
    new RxnavApi().json().rximage().search(searchCriteriaParams,'rxnav')
        .then((res) => console.log(JSON.stringify(res)))
        .catch(err => console.log(`err=${err}`))

}
//rximageSearchBySearchCriteria()

const rximageSearcher = () => {
    const rximageApi = new RxnavApi().rximage()
    const searchParams = rximageApi
    .createSearchCriteria()
    .name("atorvastatin")
    .limitNumberOfRecords(1)
    .resolution(300)
    .params()
    
    rximageApi
    .search(searchParams,'rxnav')
    .then((results)=>{
        console.log(`${JSON.stringify(results)}`)
    })
    .catch((err)=>console.log(`err=${err}`))
}
// rximageSearcher()
